local LOGO = [[
 ____  ____  _  _  ____ 
(  _ \( ___)( \( )(_  _)
 )   / )__)  )  (   )(  
(_)\_)(____)(_)\_) (__) 

@Remen for more info

]]

local PLUGIN_NAME = "Rent"
local PLUGIN_COMMAND = "Rent"
local PLUGIN_COMMAND_LC = "rent"

local THANKS = [[|
|	Special thanks go out to:
|
|		Galactica Actual - for requesting this ( https://www.vendetta-online.com/x/msgboard/3/36343 )
|
|	Enjoy!
|	
|	-----
|	
|	Customer Testimonials:
|	
|		"Before /rentTM, i was wasting almost 500k a week in un-needed reservation costs...now i pay 26,000! Thank you, /rentTM!" - Shugi Kroto
|		
|		"This is outstanding." - Galactica Actual
|
|]]

declare('Rent', Rent or {}) -- we exist!

-- ----------
-- BEGIN plugin data container
Rent = {

	-- define most of our data structures, values, and types
		version = "1.1.1", -- major ver . feature update . bug fix/minor changes
		--[[ 
			Version history:
			0.0.1 - Prototype; MVP functionality
			1.0.0 - Initial release
			1.1.0 - Large storage station support added, costs corrected, and added more data to /rent report
			1.1.1 - Fixed error at login, general code cleanup, comments, and misc minor changes
		]]--
	
	-- save file data
		systemnotes_config_save_file = 99900000999, -- used to save config data (any arbitrarily large number; 11 digits is good)
		
	-- control vars
		loggedin = false, -- is the player logged in?
		initialized = false, -- has the plugin been been initialized? (used for reloads)

	-- function declarations
		PlayerEnteredStation = {}, -- function that gets called when we login
		
	-- set up our default configuration data
		DefaultConfig = {
			debug_mode = false, -- announce on point of interest entry
			auto_reduce_active = false, -- is auto_reduce on?
		}, -- DefaultConfig

	-- active configuration data, the default is merged into this container the first time this plugin runs
	-- after that first run, the config comes from its save file
		Config = {},

	-- Station rent costs based on reservation levels
		RentLevels	=	{ 	[10000] = 0,
							[20000] = 3775,
							[30000] = 7550,
							[40000] = 11325,
							[50000] = 15100,
							[60000] = 18875,
							[70000] = 22650,
							[80000] = 26425,
							[90000] = 30200,
							[100000] = 33975,
							[110000] = 37750,
							[120000] = 41525,
							[130000] = 45300,
							[140000] = 49075,
							[150000] = 52850,
						}

		
} -- Rent

-- END plugin data container
-- ----------
-- BEGIN save/load data functions

	function Rent.SaveData()
		SaveSystemNotes(spickle(Rent.Config), Rent.systemnotes_config_save_file)
	end

-- -----

	function Rent.LoadData() 
		Rent.Config = unspickle(LoadSystemNotes(Rent.systemnotes_config_save_file))
		--  if first run, populate initial configuration with defaults
		if not Rent.Config["debug_mode"] then 
			Rent.Config = Rent.DefaultConfig
			Rent.Config["version"] = Rent.version -- seed the plugin version into the config save file
		end 
	end

-- END save/load data functions
-- ----------
-- BEGIN pretty print functions

	-- pretty print a header
	local function UTIL_Visual_BoxHeader(text)
		print(string.format("\127FFFFFF+- %-10s ---------------------------------------------------------------------------------------------------------------------\127o", PLUGIN_NAME))
		if (text) then 
			print("\127FFFFFF| \127A0A0A0 " .. text .. "\127o")
			print("\127FFFFFF+---------------------------------------------------------------------------------------------------------------------------------\127o")
		end
	end

-- -----

	-- pretty print a text line
	local function UTIL_Visual_BoxText(text)
		print ("\127FFFFFF| \127A0A0A0" .. text .. "\127o")
	end

-- -----

	-- pretty print a display line
	local function UTIL_Visual_BoxLine()
		print("\127FFFFFF+---------------------------------------------------------------------------------------------------------------------------------\127o")
	end

-- -----	

	-- pretty print a footer
	local function UTIL_Visual_BoxFooter(text)
		print("\127FFFFFF+---------------------------------------------------------------------------------------------------------------------------------\127o")
		if (text) then 
			print("\127FFFFFF| \127A0A0A0 " .. text .. "\127o")
			print("\127FFFFFF+---------------------------------------------------------------------------------------------------------------------------------\127o")
		end
	end

-- -----	

	-- pretty print left justify
	local function UTIL_PadRightString(length, to_pad)
		local c = length - string.len(to_pad)
		local padded = to_pad
		if c < 1 then return padded end
		for i = 1, c do
			padded = padded .. "  "
		end
		local skinny = string.match(to_pad, "[!1iIl,.;:]")
		if skinny then
			local x = string.len(skinny) -- skinny characters need an extra space
			if x > 0 then
				for i = 1, x do
					padded = padded .. " "
				end
			end
		end
		return padded
	end

-- -----	

	-- Utility function for adding comma to number
	local function UTIL_CommaValue(n) -- credit: http://lua-users.org/wiki/FormattingNumbers
		if not n then return 0 end
		local left,num,right = string.match(n,'^([^%d]*%d)(%d*)(.-)$')
		return left..(num:reverse():gsub('(%d%d%d)','%1,'):reverse())..right
	end

-- END pretty print functions
-- ----------
-- BEGIN COMM functions
	
	local function UTIL_SendToDebug(message)
		if Rent.Config.debug then print("\127FFA0A0" .. PLUGIN_COMMAND .. " DEBUG:\127A0A0A0 " .. message .. "\127o") end
	end

-- END COMM functions
-- ----------
-- BEGIN plugin functions

function Rent.AutoReduce()
	if not Rent.Config.auto_reduce_active then return end -- autoreduce disabled
	if PlayerInStation() and GetStationType() then return end -- we're in a capship
	if GetStationMaxCargo() == 10000 then return end -- we're already as low as we can go
	local unrent = math.floor((GetStationMaxCargo()-GetStationCurrentCargo())/10000)
	if unrent > 0 then 
		print(string.format("\12700FF00Station storage rent reduced to %s to match current inventory storage requirement of %scu.", GetStationMaxCargo() - (unrent * 10000), GetStationCurrentCargo()))
		UnrentStorage(unrent * 10000) -- unrent storage down to required level
	end
end

local function RentReport()
	local report = ""
	local Stations = {}
	local total_rent = 0
	local total_savings = 0

	UTIL_Visual_BoxHeader("Location \t\tStored \t\tReserved \t\tCost \t\tNote")

	local sectors = 31 * 256
	for sector = 1, sectors do
		for station = 1, 5 do -- 5 stations?! Maybe, who knows.
			local stationid = sector * 256 + station
			if GetStationMaxCargo(stationid) > 10000 then 
				Stations[GetStationName(stationid)] = GetStationMaxCargo(stationid)
				-- local lname ShortLocationStr(sector)
				local reservation = GetStationMaxCargo(stationid)
				local utilized = GetStationCurrentCargo(stationid)
				local sname = AbbrLocationStr(sector)
				
				-- maths are hard
				local cost = Rent.RentLevels[reservation] -- rent based on cargo reservation
				local unrent = math.floor((reservation - utilized)/10000)
				local rent_diff = reservation - (unrent * 10000)
				local overpaid = cost - Rent.RentLevels[rent_diff]
				total_rent = total_rent + cost
				total_savings = total_savings + overpaid
				
				-- display stuff
				local flag = ""
				if unrent > 0 then flag = "\127FF0000** Reservation too high, overspend: " .. UTIL_CommaValue(overpaid) .. "c!\127A0A0A0" end
				UTIL_Visual_BoxText(sname .. " \t\t" .. UTIL_PadRightString(8, utilized .. "cu") .. " \t\t" .. reservation .. "cu \t\t" .. UTIL_CommaValue(cost) .. "c \t" .. flag)
			end
		end
	end
	UTIL_Visual_BoxLine()
	local flag = ""
	if total_savings > 0 then flag = " --\127FF0000 [ " .. UTIL_CommaValue(total_savings) .. "c overspent ]\127o" end
	UTIL_Visual_BoxText("Estimated rent: " .. UTIL_CommaValue(total_rent) .. "c/week" .. flag )
	UTIL_Visual_BoxFooter("'/rent autoreduce [on|off]' toggles auto-reduction of rented storage at docking, based on inventory levels.")
end


-- END plugin functions
-- ----------
-- BEGIN parser

	-- /COMMAND parser
	function Rent.SlashCommandParser(data, args)

		if args and args[1] then

			local lc_command = string.lower(args[1])

			-- sub command parser
			if (lc_command == "autoreduce") then 
				-- sub commands
				if args[2] then
					local lc_sub_command = string.lower(args[2])
					if lc_sub_command == "on" then print("\127FFFFFFRent autoreduce \12700FF00ON\127o"); Rent.Config.auto_reduce_active=true; Rent.SaveData(); return end
					if lc_sub_command == "off" then print("\127FFFFFFRent autoreduce \127FF0000OFF\127o"); Rent.Config.auto_reduce_active=false; Rent.SaveData(); return end
					if lc_sub_command == "force" then print("\127FFFFFFRent autoreduce forced.\127o"); Rent.AutoReduce(); return end
					-- none of these? fall through and show some help text...
				end

				UTIL_Visual_BoxText("Usage:")
				UTIL_Visual_BoxText("-- '/rent autoreduce [on|off]' will toggle reduction of rented space at docked stations based on current inventory levels.")
				return
			end

			if lc_command == "credits" then 
				UTIL_Visual_BoxHeader("Welcome to " .. PLUGIN_NAME .. ", version " .. Rent.version)
				print("\127FFFFFF" .. THANKS .. "\127o")
				UTIL_Visual_BoxFooter("For more information, @Remen.")
			end

			if lc_command == "help" then 
				UTIL_Visual_BoxHeader("Welcome to " .. PLUGIN_NAME .. ", version " .. Rent.version)
				UTIL_Visual_BoxText("Usage:")
				UTIL_Visual_BoxText("-- '" .. PLUGIN_COMMAND_LC .. "' will show your current inventory levels and rent estimate.")
				UTIL_Visual_BoxText("-- '" .. PLUGIN_COMMAND_LC .. " autoreduce [on|off|force]' will toggle (or force) reduction of rented space at docked station based on inventory level.")
				UTIL_Visual_BoxFooter()
			end
		end
		
		-- we're going to default to showing the rent report. 
		RentReport()

	end

-- END parser
-- ----------
-- BEGIN plugin initialization

	-- we separate this out so we can call it at plugin restart, or at login time. one uses an event the other checks to see if we're already in game.
	function Rent.BootPlugin()

		-- we only ever want to do this once at login, not at every time we dock.
		if not Rent.loggedin then
			Rent.LoadData()
			if Rent.Config.auto_reduce_active then 
				print ("\127FFFFFFRent AutoReduce is \12700FF00enabled\127FFFFFF.")
			else 
				print ("\127FFFFFFRent AutoReduce is \127FF0000disabled\127FFFFFF.")
			end
			Rent.loggedin = true
		end

	end

-- -----

	-- Boot the plugin, and populate some key data that can't be populated without a player being logged in
	-- this is called by the registered event from initilization
	function Rent.PlayerEnteredStation(event, data)
		if Rent.loggedin then
			Rent.AutoReduce()
		else -- we haven't booted the plugin yet
			Rent.BootPlugin()
		end
	end

-- -----

	-- this is all executed at plugin load time (usually game start, but also at ReloadInterface())
	-- locals rule, globals SUCK! Anti-global is the way to go!
	local init; do

		-- Plugin command parser
		RegisterUserCommand(PLUGIN_COMMAND_LC, Rent.SlashCommandParser)

		-- A little something for the errorlog at game load time
		console_print(LOGO) -- bragalicious

		-- check to see if we're logged in (RestartConsole() used) or trap log in event
		if not GetPlayerName() then
			-- we can't load data yet, because we don't have access to a player account, so, hook the ENTERED_STATION event and watch for login.
			RegisterEvent(Rent.PlayerEnteredStation, "ENTERED_STATION")
			print("\127FFFFFF" .. PLUGIN_COMMAND .. " INIT\127A0A0A0: Plugin pre-loaded; command '/" .. PLUGIN_COMMAND_LC .. "' registered; hooks registered... waiting for login.")
		else
			-- Otherwise, this was a ReloadInterface(), so let's go ahead and launch our plugin
			Rent.BootPlugin()
			print("\127FFFFFF" .. PLUGIN_COMMAND .. " INIT\127A0A0A0: Plugin loaded. '/" .. PLUGIN_COMMAND_LC .. "' for more info.")
		end

	end -- init

-- END plugin initialization
-- ----------
-- END PLUGIN